/* GSKrab.m - this file is part of GSKrab
 *
 * Copyright (C) 2006 Wolfgang Sourdeau
 *
 * Author: Wolfgang Sourdeau <Wolfgang@Contre.COM>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#import <Foundation/NSAutoreleasePool.h>
#import <Foundation/NSBundle.h>
#import <Foundation/NSConnection.h>
#import <Foundation/NSDistantObject.h>
#import <Foundation/NSProcessInfo.h>
#import <Foundation/NSRunLoop.h>
#import <Foundation/NSTask.h>

#import "GSKrabManager.h"

static BOOL
ServerRunning ()
{
  NSDistantObject <GSKrabManagerControl> *server;
  BOOL result;

  server
    = [NSConnection rootProxyForConnectionWithRegisteredName: @"GSKrabServer"
                    host: nil];
  result = (server != nil);

  return result;
}

static int
StopDaemon ()
{
  NSDistantObject <GSKrabManagerControl> *server;
  int rc;

  server
    = [NSConnection rootProxyForConnectionWithRegisteredName: @"GSKrabServer"
                    host: nil];
  if (server)
    {
      [server setProtocolForProxy: @protocol (GSKrabManagerControl)];
      [server stop];
      rc = 0;
    }
  else
    {
      NSLog (@"server not reachable (not running?)");
      rc = 2;
    }

  return rc;
}

static int
SpawnDaemon ()
{
  NSTask *daemon;
  NSProcessInfo *processInfo;
  NSFileHandle *nullDevice;
  NSMutableArray *arguments;

  processInfo = [NSProcessInfo processInfo];
  daemon = [NSTask new];
  arguments = [NSMutableArray new];
  [daemon setLaunchPath: [[NSBundle mainBundle] executablePath]];
  [arguments addObjectsFromArray: [processInfo arguments]];
  [arguments removeObjectAtIndex: 0];
  [arguments addObject: @"--no-fork"];
  [daemon setArguments: arguments];
  [daemon setEnvironment: [processInfo environment]];
  nullDevice = [NSFileHandle fileHandleWithNullDevice];
  [daemon setStandardInput: nullDevice];
  [daemon setStandardOutput: nullDevice];
  [daemon setStandardError: nullDevice];

  [daemon launch];

  return ([daemon isRunning] ? 0 : [daemon terminationStatus]);
}

int
main (int argc, char *argv[], char *env[])
{
  NSArray *arguments;
  NSAutoreleasePool *pool;
  GSKrabManager *krabManager;
  NSProcessInfo *processInfo;
  int rc;

  pool = [NSAutoreleasePool new];

  [NSProcessInfo initializeWithArguments: argv
                 count: argc
                 environment: env];
  processInfo = [NSProcessInfo processInfo];
  arguments = [processInfo arguments];

  if ([arguments containsObject: @"--no-fork"])
    {
      [processInfo setProcessName: @"GSKrab"];
      krabManager = [GSKrabManager new];
      if (krabManager)
        {
          [[NSRunLoop currentRunLoop] run];
          rc = 0;
        }
      else
        rc = 1;
    }
  else if ([arguments containsObject: @"--stop"])
    rc = StopDaemon ();
  else if ([arguments containsObject: @"--running"])
    rc = (ServerRunning() ? 0 : 1);
  else
    {
      if (ServerRunning())
        {
          NSLog (@"server already running");
          rc = 1;
        }
      else
        rc = SpawnDaemon ();
    }

  [pool release];

  return rc;
}
