/* GSKrabManager.h - this file is part of GSKrab
 *
 * Copyright (C) 2006 Wolfgang Sourdeau
 *
 * Author: Wolfgang Sourdeau <Wolfgang@Contre.COM>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef GSKRAB_H
#define GSKRAB_H

#import <Foundation/NSMapTable.h>
#import <Foundation/NSObject.h>

#import "GSKrabDesktopInterface.h"

@class NSConnection;
@class NSNotification;
@class NSPort;
@class NSTimer;

@class GSKrabServer;

@protocol GSKrabDesktopInterface;

@protocol GSKrabClient

- (void) performActionForKey: (GSKrabKeyCode) krabKeyCode;

@end

@protocol GSKrabManagerControl

- (void) stop;

@end

@interface GSKrabManager : NSObject
{
  NSConnection *serverConnection;
  NSMapTable *table;
  NSTimer *keyTimer;
  NSObject <GSKrabDesktopInterface> *interface;
  GSKrabServer *server;
  NSMapTable *keyTable;
  NSMutableArray *applications;
}

- (void) registerKey: (GSKrabKeyCode) krabKeyCode
           forObject: (NSObject *) object;
- (BOOL) handleKeyPress: (GSKrabKeyCode) krabKeyCode;

- (void) registerApplication: (NSString *) application
             withServiceName: (NSString *) name;
- (void) applicationWithServiceName: (NSString *) serviceName
                subscribesToKeyCode: (GSKrabKeyCode) krabKeyCode;
- (void) stop;

@end

#endif /* GSKRAB_H */
