/* GSKrabServer.m - this file is part of GSKrab
 *
 * Copyright (C) 2006 Wolfgang Sourdeau
 *
 * Author: Wolfgang Sourdeau <Wolfgang@Contre.COM>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#import <Foundation/NSConnection.h>
#import <Foundation/NSTimer.h>

#import "GSKrabManager.h"
#import "GSKrabServer.h"

@implementation GSKrabServer

- (id) init
{
  if ((self = [super init]))
    {
      manager = nil;
      clients = [NSMutableDictionary new];
    }

  return self;
}

- (void) setManager: (GSKrabManager *) newManager
{
  manager = newManager;
}

- (NSString *) registerAsClient: (NSString *) name
                fromApplication: (NSString *) application
{
  NSString *identifier;

  identifier = name;
  [clients setObject: name forKey: identifier];
  [manager registerApplication: application withServiceName: name];

  return identifier;
}

- (void) subscribeToKeyCode: (GSKrabKeyCode) krabKeyCode
             withIdentifier: (NSString *) identifier
{
  NSString *name;

//   NSLog (@"'%@' subscribing to %d", identifier, krabKeyCode);
  name = [clients objectForKey: identifier];
  if (name)
    [manager applicationWithServiceName: name subscribesToKeyCode: krabKeyCode];
  else
    NSLog (@"name not found for '%@'", identifier);
}

- (void) stop
{
  [NSTimer scheduledTimerWithTimeInterval: 0.1
           target: manager
           selector: @selector (stop)
           userInfo: nil
           repeats: NO];
}

- (void) dealloc
{
  [clients release];
}

@end
