/* GSKrabX11Interface.m - this file is part of GSKrab
 *
 * Copyright (C) 2006 Wolfgang Sourdeau
 *
 * Author: Wolfgang Sourdeau <Wolfgang@Contre.COM>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#import <Foundation/NSAutoreleasePool.h>
#import <Foundation/NSString.h>
#import <Foundation/NSThread.h>
#import <Foundation/NSValue.h>

#import "GSKrabDesktopInterface.h"
#import "GSKrabManager.h"
#import "GSKrabX11Interface.h"

static int (*PreviousErrorHandler) (Display *xDisplay, XErrorEvent *error);

static int
ErrorHandler (Display *xDisplay, XErrorEvent *error)
{
  int rc;

  NSLog (@"error");
  if (error->request_code == X_GrabKey)
    {
      NSLog (@"failure grabbing some key (%d, %d, %d)\n",
             error->resourceid, error->error_code, error->minor_code);
      rc = 0;
    }
  else
    rc = PreviousErrorHandler (xDisplay, error);

  return rc;
}

@implementation GSKrabX11Interface

+ (id) sharedInterface
{
  static GSKrabX11Interface *sharedInterface = nil;

  if (!sharedInterface)
    sharedInterface = [self new];

  return sharedInterface;
}

- (void) _initX11
{
  X11KeyEquivalent equivs[] =
    {
      {@"XF86AddFavorite", GSKrabNone},
      {@"XF86ApplicationLeft", GSKrabNone},
      {@"XF86ApplicationRight", GSKrabNone},
      {@"XF86AudioLowerVolume", GSKrabVolumeDecrease},
      {@"XF86AudioMedia", GSKrabNone},
      {@"XF86AudioMute", GSKrabVolumeMute},
      {@"XF86AudioNext", GSKrabTrackNext},
      {@"XF86AudioPause", GSKrabPlay},
      {@"XF86AudioPlay", GSKrabPlay},
      {@"XF86AudioPrev", GSKrabTrackPrevious},
      {@"XF86AudioRaiseVolume", GSKrabVolumeIncrease},
      {@"XF86AudioRecord", GSKrabNone},
      {@"XF86AudioRewind", GSKrabNone},
      {@"XF86AudioStop", GSKrabStop},
      {@"XF86Away", GSKrabNone},
      {@"XF86Back", GSKrabNone},
      {@"XF86BackForward", GSKrabNone},
      {@"XF86Book", GSKrabNone},
      {@"XF86BrightnessAdjust", GSKrabNone},
      {@"XF86Calculator", GSKrabNone},
      {@"XF86Calendar", GSKrabNone},
      {@"XF86CD", GSKrabNone},
      {@"XF86Clear", GSKrabNone},
      {@"XF86Close", GSKrabNone},
      {@"XF86Community", GSKrabNone},
      {@"XF86ContrastAdjust", GSKrabNone},
      {@"XF86Copy", GSKrabNone},
      {@"XF86Cut", GSKrabNone},
      {@"XF86Display", GSKrabNone},
      {@"XF86Documents", GSKrabNone},
      {@"XF86DOS", GSKrabNone},
      {@"XF86Eject", GSKrabEject},
      {@"XF86Excel", GSKrabNone},
      {@"XF86Explorer", GSKrabNone},
      {@"XF86Favorites", GSKrabNone},
      {@"XF86Finance", GSKrabNone},
      {@"XF86Forward", GSKrabNone},
      {@"XF86Game", GSKrabNone},
      {@"XF86Go", GSKrabNone},
      {@"XF86History", GSKrabNone},
      {@"XF86HomePage", GSKrabNone},
      {@"XF86HotLinks", GSKrabNone},
      {@"XF86Launch0", GSKrabNone},
      {@"XF86Launch1", GSKrabNone},
      {@"XF86Launch2", GSKrabNone},
      {@"XF86Launch3", GSKrabNone},
      {@"XF86Launch4", GSKrabNone},
      {@"XF86Launch5", GSKrabNone},
      {@"XF86LaunchA", GSKrabNone},
      {@"XF86LaunchB", GSKrabNone},
      {@"XF86LaunchC", GSKrabNone},
      {@"XF86LightBulb", GSKrabNone},
      {@"XF86LogOff", GSKrabNone},
      {@"XF86Mail", GSKrabMail},
      {@"XF86MailForward", GSKrabNone},
      {@"XF86Market", GSKrabNone},
      {@"XF86Meeting", GSKrabNone},
      {@"XF86MenuKB", GSKrabNone},
      {@"XF86MenuPB", GSKrabNone},
      {@"XF86Messenger", GSKrabNone},
      {@"XF86Music", GSKrabNone},
      {@"XF86MyComputer", GSKrabNone},
      {@"XF86MySites", GSKrabNone},
      {@"XF86New", GSKrabNone},
      {@"XF86News", GSKrabNone},
      {@"XF86OfficeHome", GSKrabNone},
      {@"XF86Open", GSKrabNone},
      {@"XF86OpenURL", GSKrabNone},
      {@"XF86Option", GSKrabNone},
      {@"XF86Paste", GSKrabNone},
      {@"XF86Phone", GSKrabNone},
      {@"XF86Pictures", GSKrabNone},
      {@"XF86PowerOff", GSKrabNone},
      {@"XF86Q", GSKrabNone},
      {@"XF86Refresh", GSKrabNone},
      {@"XF86Reload", GSKrabNone},
      {@"XF86Reply", GSKrabNone},
      {@"XF86RotateWindows", GSKrabNone},
      {@"XF86RotationKB", GSKrabNone},
      {@"XF86RotationPB", GSKrabNone},
      {@"XF86Save", GSKrabNone},
      {@"XF86ScreenSaver", GSKrabNone},
      {@"XF86ScrollClick", GSKrabNone},
      {@"XF86ScrollDown", GSKrabNone},
      {@"XF86ScrollUp", GSKrabNone},
      {@"XF86Search", GSKrabNone},
      {@"XF86Send", GSKrabNone},
      {@"XF86Shop", GSKrabNone},
      {@"XF86Sleep", GSKrabNone},
      {@"XF86Spell", GSKrabNone},
      {@"XF86SplitScreen", GSKrabNone},
      {@"XF86Standby", GSKrabNone},
      {@"XF86Start", GSKrabNone},
      {@"XF86Stop", GSKrabNone},
      {@"XF86Support", GSKrabNone},
      {@"XF86TaskPane", GSKrabNone},
      {@"XF86Terminal", GSKrabNone},
      {@"XF86Tools", GSKrabNone},
      {@"XF86Travel", GSKrabNone},
      {@"XF86User1KB", GSKrabNone},
      {@"XF86User2KB", GSKrabNone},
      {@"XF86UserPB", GSKrabNone},
      {@"XF86VendorHome", GSKrabNone},
      {@"XF86Video", GSKrabNone},
      {@"XF86WakeUp", GSKrabNone},
      {@"XF86WebCam", GSKrabNone},
      {@"XF86Word", GSKrabNone},
      {@"XF86WWW", GSKrabWWW},
      {@"XF86Xfer", GSKrabNone},
      {@"XF86ZoomIn", GSKrabNone},
      {@"XF86ZoomOut", GSKrabNone},
      {nil, GSKrabNone}
    };
  X11KeyEquivalent *currentEquiv;

  currentEquiv = equivs;
  while (currentEquiv->keyName)
    {
      if (currentEquiv->keyCode != GSKrabNone)
        NSMapInsert (X11KeyTable, (void *) (currentEquiv->keyCode),
                     currentEquiv->keyName);
      currentEquiv++;
    }

  _GSKRAB_THREAD_RUNNING
    = XInternAtom (xDisplay, "_GSKRAB_THREAD_RUNNING", NO);

  rootWindow = DefaultRootWindow (xDisplay);

  XSelectInput (xDisplay, rootWindow, KeyPressMask | PropertyChangeMask);
}

- (id) init
{
  if ((self = [super init]))
    {
      XInitThreads ();
      xDisplay = XOpenDisplay (NULL);
      if (!xDisplay)
        {
          NSLog (@"Unable to open display");
          [self release];
          self = nil;
        }
      else
        {
          PreviousErrorHandler = XSetErrorHandler (ErrorHandler);
          if (!PreviousErrorHandler)
            NSLog (@"Did not register new error handler");
          bindingTable = NSCreateMapTable (NSIntMapKeyCallBacks,
                                           NSIntMapValueCallBacks,
                                           GSKrabMaxNumberOfKeys);
          X11KeyTable = NSCreateMapTable (NSIntMapKeyCallBacks,
                                          NSObjectMapValueCallBacks,
                                          GSKrabMaxNumberOfKeys);
          [self _initX11];
        }
    }

  return self;
}

- (void) dealloc
{
  XCloseDisplay (xDisplay);
  NSFreeMapTable (X11KeyTable);
  NSFreeMapTable (bindingTable);
  [super dealloc];
}

- (void) setManager: (GSKrabManager *) krabManager
{
  manager = krabManager;
}

- (void) _registerKeyCode: (GSKrabKeyCode) krabKeyCode
               doRegister: (BOOL) doRegister
{
  NSString *keyName;
  KeySym keysymbol;
  KeyCode x11KeyCode;
  int keycode;

  keyName = (NSString *) NSMapGet (X11KeyTable, (void *) krabKeyCode);
  if (keyName)
    {
      keysymbol = XStringToKeysym ([keyName cString]);
      if (keysymbol != NoSymbol)
        {
          x11KeyCode = XKeysymToKeycode (xDisplay, keysymbol);
          if (x11KeyCode != NoSymbol)
            {
              keycode = x11KeyCode;
              XLockDisplay (xDisplay);
              if (doRegister)
                {
                  XGrabKey (xDisplay, keycode, AnyModifier, rootWindow, YES,
                            GrabModeAsync, GrabModeAsync);
                  NSMapInsert (bindingTable,
                               (void *) keycode, (void *) krabKeyCode);
                }
              else
                {
                  XUngrabKey (xDisplay, keycode, AnyModifier, rootWindow);
                  NSMapRemove (bindingTable, (void *) keycode);
                }
              XUnlockDisplay (xDisplay);
            }
        }
    }
}

- (void) registerKeyCode: (GSKrabKeyCode) krabKeyCode
{
  [self _registerKeyCode: krabKeyCode doRegister: YES];
}

- (void) unregisterKeyCode: (GSKrabKeyCode) krabKeyCode
{
  [self _registerKeyCode: krabKeyCode doRegister: NO];
}

- (BOOL) _monitoringFlag
{
  Atom TypeReturn;
  int FormatReturn;
  unsigned long NumItemsReturn;
  unsigned long BytesAfterReturn;
  unsigned char *PropertyValue;

  XGetWindowProperty (xDisplay, rootWindow, _GSKRAB_THREAD_RUNNING, 0,
                      sizeof (BOOL) * 8, NO, AnyPropertyType,
                      &TypeReturn,
                      &FormatReturn, &NumItemsReturn, &BytesAfterReturn,
                      &PropertyValue);

  return (TypeReturn != None);
}

- (void) _handleKeyPress: (NSNumber *) krabKeyCode
{
  [manager handleKeyPress: [krabKeyCode intValue]];
}

- (void) _XLoop
{
  XEvent xEvent;
  int rc;
  NSAutoreleasePool *pool;
  GSKrabKeyCode krabKeyCode;

  pool = [NSAutoreleasePool new];

  while ([self _monitoringFlag])
    {
      rc = XMaskEvent (xDisplay, KeyPressMask | PropertyChangeMask, &xEvent);
      if (xEvent.type == KeyPress)
        {
          krabKeyCode
            = (GSKrabKeyCode) NSMapGet (bindingTable,
                                        (void *) xEvent.xkey.keycode);
          [self performSelectorOnMainThread: @selector (_handleKeyPress:)
                withObject: [NSNumber numberWithInt: krabKeyCode]
                waitUntilDone: YES];
        }
    }

  [pool release];

  [NSThread exit];
}

- (void) start
{
  BOOL running;

  running = YES;
  XLockDisplay (xDisplay);
  if ([self _monitoringFlag])
    XDeleteProperty (xDisplay, rootWindow, _GSKRAB_THREAD_RUNNING);
  XChangeProperty (xDisplay, rootWindow, _GSKRAB_THREAD_RUNNING,
                   XA_INTEGER, sizeof (BOOL) * 8, PropModeAppend,
                   &running, 1);
  XUnlockDisplay (xDisplay);

  [NSThread detachNewThreadSelector: @selector (_XLoop)
            toTarget: self
            withObject: nil];
}

- (void) stop
{
  XLockDisplay (xDisplay);
  if ([self _monitoringFlag])
    XDeleteProperty (xDisplay, rootWindow, _GSKRAB_THREAD_RUNNING);
  XUnlockDisplay (xDisplay);
  while ([self _monitoringFlag])
    [NSThread sleepUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.01]];
}

@end
